"""
distutilazy.test
-----------------

command classes to help run tests

:license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os
from os.path import abspath, basename, dirname
import sys
import fnmatch
from importlib import import_module
import unittest
from distutils.core import Command
from types import ModuleType

__version__ = "0.3.0"


def test_suite_for_modules(modules):
    suite = unittest.TestSuite()
    test_loader = unittest.defaultTestLoader
    for module in modules:
        module_tests = test_loader.loadTestsFromModule(module)
        suite.addTests(module_tests)
    return suite


class RunTests(Command):
    description = """Run test suite"""
    user_options = [("root=", "r", "path to tests suite dir"),
                    ("pattern=", "p", "test file name pattern"),
                    ("verbosity=", "v", "verbosity level [1,2,3]"),
                    ("files=", None,
                     "run specified test files (comma separated)")]

    def initialize_options(self):
        self.root = os.path.join(os.getcwd(), 'tests')
        self.pattern = "test*.py"
        self.verbosity = 1
        self.files = None

    def finalize_options(self):
        if not os.path.exists(self.root):
            raise IOError("Failed to access root path '{}'".format(self.root))
        verbosity = min(int(self.verbosity), 3)
        if verbosity < 1:
            self.verbosity = 1
        else:
            self.verbosity = verbosity
        if self.files:
            self.files = map(lambda name: name.strip(), self.files.split(','))

    def get_modules_from_files(self, files):
        modules = []
        for file_name in files:
            directory = dirname(file_name)
            package_name = self._import_dir_as_package(directory)
            if not package_name:
                sys.path.insert(0, directory)
            module_name, _, extension = basename(file_name).rpartition('.')
            if not module_name:
                self.announce(
                    "failed to find module name from filename '{}'." +
                    "skipping this file".format(file_name))
                continue
            if package_name:
                module_name = '.' + module_name
            self.announce(
                "importing module '{}' from file '{}' ...".format(module_name,
                                                                  file_name))
            module = import_module(module_name, package=package_name)
            modules.append(module)
        return modules

    def _import_dir_as_package(self, directory, ensure=False):
        package_name = basename(directory)
        abs_dir = abspath(directory)
        if package_name:
            try:
                self.announce(
                    "importing '{0}' as package ...".format(package_name))
                package = import_module(package_name)
                if ensure and hasattr(package, '__path__') and \
                            abspath(package.__path__[0]) != abs_dir:
                    raise ImportError(
                        "directory '{}' is not a package to import".format(
                            abs_dir))
            except ImportError as err:
                self.announce(
                    "failed to import '{}'. {}".format(package_name, err))
                package_name = None
        return package_name

    def find_test_modules_from_package_path(self, package_path):
        """Import and return modules from package __all__,
        if path is found to be a package.
        """
        package_dir = dirname(package_path)
        package_name = basename(package_path)
        if package_dir:
            sys.path.insert(0, package_dir)
        try:
            self.announce(
                "importing package '{}' ...".format(package_name)
            )
            package = import_module(package_name)
            if package and hasattr(package, '__all__'):
                modules = []
                for module_name in package.__all__:
                    module = import_module('{}.{}'.format(
                        package_name, module_name))
                    if type(module) == ModuleType \
                            and module not in modules:
                        modules.append(module)
                return modules
        except ImportError as err:
            self.announce(
                "failed to import '{}'. not a package. {}".format(
                    package_name, err))
        return []

    def find_test_modules_from_test_files(self, root, pattern):
        """Return list of test modules from the the files in the path
        whose name match the pattern
        """
        modules = []
        abs_root = abspath(root)
        for (dir_path, directories, file_names) in os.walk(abs_root):
            package_name = self._import_dir_as_package(dir_path)
            if not package_name:
                sys.path.insert(0, dir_path)
            for filename in fnmatch.filter(file_names, pattern):
                module_name, _, extension = basename(filename).rpartition('.')
                if not module_name:
                    self.announce(
                        "failed to find module name from filename '{}'." +
                        "skipping this test".format(filename))
                    continue
                if package_name:
                    module_name = '.' + module_name
                self.announce(
                    "importing module '{}' from '{}' ...".format(
                        module_name, filename
                    )
                )
                try:
                    module = import_module(module_name, package_name)
                    if type(module) == ModuleType and module not in modules:
                        modules.append(module)
                except (ImportError, ValueError, SystemError) as ex:
                    self.announce(
                        "failed to import '{}' from '{}'. {}." +
                        "skipping this file!".format(module_name, filename, ex)
                    )
        return modules

    def get_test_runner(self):
        return unittest.TextTestRunner(verbosity=self.verbosity)

    def run(self):
        if self.files:
            modules = self.get_modules_from_files(self.files)
        else:
            self.announce("searching for test package modules ...")
            modules = self.find_test_modules_from_package_path(self.root)
            if not modules:
                self.announce("searching for test files ...")
                modules = self.find_test_modules_from_test_files(self.root,
                                                                 self.pattern)
        if not modules:
            self.announce("found no test files")
            return False
        suite = test_suite_for_modules(modules)
        runner = self.get_test_runner()
        self.announce("running tests ...")
        runner.run(suite)

run_tests = RunTests



"""
distutilazy.tests.test_pyinstaller
----------------------------------

Tests for distutilazy.pyinstaller module.

:license: MIT, see LICENSE for more details.
"""

import sys
from os.path import dirname
from unittest import TestCase, main
import re

here = dirname(__file__)
sys.path.insert(0, dirname(here))
sys.path.insert(0, here)

from distutilazy.pyinstaller import BdistPyInstaller, CleanAll
from distutils.dist import Distribution


class TestPyinstaller(TestCase):

    def test_finalize_opts(self):
        dist = Distribution()
        pi = BdistPyInstaller(dist)
        pi.target = "fake.py"
        pi.finalize_options()
        self.assertTrue( re.match(".+", pi.name) )
        self.assertTrue(pi.pyinstaller_opts)

    def test_clean_all(self):
        dist = Distribution()
        cl = CleanAll(dist)
        cl.finalize_options()
        paths = cl.get_extra_paths()
        self.assertTrue(paths)
        spec = paths.pop()
        self.assertTrue( re.match("\S+\.spec", spec) )

if __name__ == "__main__":
    main()


"""
distutilazy.tests.test_test
----------------------------

Tests for distutilazy.test module

:license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import sys
from os.path import dirname, basename
from distutils.dist import Distribution
from unittest import TestCase, TestSuite, main

here = dirname(__file__)
sys.path.insert(0, dirname(here))
sys.path.insert(0, here)

from distutilazy.test import RunTests, test_suite_for_modules

__file__ = basename(__file__[:-1] if __file__.endswith('.pyc') else __file__)


def get_module_names(modules):
    return map(lambda m: m.__name__, modules)


class TestTest(TestCase):

    def test_find_modules_from_package_path(self):
        dist = Distribution()
        test_runner = RunTests(dist)
        test_runner.finalize_options()
        modules = test_runner.find_test_modules_from_package_path(here)
        self.assertIn('tests.test_test', get_module_names(modules))

    def test_get_modules_from_files(self):
        dist = Distribution()
        test_runner = RunTests(dist)
        test_runner.finalize_options()
        self.assertEqual(
            [], test_runner.get_modules_from_files(['none_existing_file']))
        modules = test_runner.get_modules_from_files([__file__])
        self.assertEqual(1, len(modules))
        self.assertEqual('test_test', modules.pop().__name__)

    def test_find_test_modules_from_test_files(self):
        dist = Distribution()
        test_runner = RunTests(dist)
        test_runner.finalize_options()
        modules = test_runner.find_test_modules_from_test_files(
            here, 'none_exiting_pattern')
        self.assertEqual([], modules)
        modules = test_runner.find_test_modules_from_test_files(here, __file__)
        self.assertEqual(1, len(modules))
        self.assertEqual('tests.test_test', modules.pop().__name__)
        modules = test_runner.find_test_modules_from_test_files(here, 'test_*')
        module_names = get_module_names(modules)
        self.assertIn('tests.test_test', module_names)
        self.assertIn('test_subdir', module_names)

    def test_test_suite_for_modules(self):
        self.assertIsInstance(test_suite_for_modules([]), TestSuite)

    def test_get_test_runner(self):
        dist = Distribution()
        test_runner = RunTests(dist)
        test_runner.finalize_options()
        runner = test_runner.get_test_runner()
        self.assertTrue(hasattr(runner, 'run'))
        self.assertTrue(hasattr(runner.run, '__call__'))

if __name__ == '__main__':
    main()



"""
This file is a data fixture for test_test.py
it is supposed to be discovered and loaded as
as test module in recursive search for test files.
This file is not meant to be imported, nor to implement
any feature.
"""



"""
distutilazy.command.clean_pyc
-----------------------------

Command to clean compiled python files

:license: MIT, see LICENSE for more details.
"""

import distutilazy.clean


class clean_pyc(distutilazy.clean.CleanPyc):
    pass


"""
distutilazy.util
----------------
utility functions

:license: MIT, see LICENSE for more details.
"""

import os
import fnmatch


def find_files(root, pattern):
    """Find all files matching the glob pattern recursively

    :param root: string
    :param pattern: string
    :return: list of file paths relative to root
    """
    results = []
    for base, dirs, files in os.walk(root):
        matched = fnmatch.filter(files, pattern)
        results.extend(os.path.join(base, f) for f in matched)
    return results


def find_directories(root, pattern):
    """Find all directories matching the glob pattern recursively

    :param root: string
    :param pattern: string
    :return: list of dir paths relative to root
    """
    results = []
    for base, dirs, files in os.walk(root):
        matched = fnmatch.filter(dirs, pattern)
        results.extend(os.path.join(base, d) for d in matched)
    return results



"""
distutilazy.command.clean_all
-----------------------------

Command to clean all temporary files

:license: MIT, see LICENSE for more details.
"""

import distutilazy.pyinstaller


class clean_all(distutilazy.pyinstaller.CleanAll):
    pass
